# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast
from typing_extensions import overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.zones import setting_edit_params
from ..._base_client import make_request_options
from ...types.zones.setting_get_response import SettingGetResponse
from ...types.zones.setting_edit_response import SettingEditResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    @overload
    def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        """
        Updates a single zone setting by the identifier

        Args:
          zone_id: Identifier

          setting_id: Setting name

          enabled: ssl-recommender enrollment setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        value: setting_edit_params.Variant1Value | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        """
        Updates a single zone setting by the identifier

        Args:
          zone_id: Identifier

          setting_id: Setting name

          value: Value of the zone setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id"])
    def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        enabled: bool | Omit = omit,
        value: setting_edit_params.Variant1Value | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return cast(
            Optional[SettingEditResponse],
            self._patch(
                path_template("/zones/{zone_id}/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
                body=maybe_transform(
                    {
                        "enabled": enabled,
                        "value": value,
                    },
                    setting_edit_params.SettingEditParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[SettingEditResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[SettingEditResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get(
        self,
        setting_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingGetResponse]:
        """
        Fetch a single zone setting by name

        Args:
          zone_id: Identifier

          setting_id: Setting name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return cast(
            Optional[SettingGetResponse],
            self._get(
                path_template("/zones/{zone_id}/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[SettingGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[SettingGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    @overload
    async def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        enabled: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        """
        Updates a single zone setting by the identifier

        Args:
          zone_id: Identifier

          setting_id: Setting name

          enabled: ssl-recommender enrollment setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        value: setting_edit_params.Variant1Value | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        """
        Updates a single zone setting by the identifier

        Args:
          zone_id: Identifier

          setting_id: Setting name

          value: Value of the zone setting.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id"])
    async def edit(
        self,
        setting_id: str,
        *,
        zone_id: str,
        enabled: bool | Omit = omit,
        value: setting_edit_params.Variant1Value | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingEditResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return cast(
            Optional[SettingEditResponse],
            await self._patch(
                path_template("/zones/{zone_id}/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
                body=await async_maybe_transform(
                    {
                        "enabled": enabled,
                        "value": value,
                    },
                    setting_edit_params.SettingEditParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[SettingEditResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[SettingEditResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get(
        self,
        setting_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SettingGetResponse]:
        """
        Fetch a single zone setting by name

        Args:
          zone_id: Identifier

          setting_id: Setting name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not setting_id:
            raise ValueError(f"Expected a non-empty value for `setting_id` but received {setting_id!r}")
        return cast(
            Optional[SettingGetResponse],
            await self._get(
                path_template("/zones/{zone_id}/settings/{setting_id}", zone_id=zone_id, setting_id=setting_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[SettingGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[SettingGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.edit = to_raw_response_wrapper(
            settings.edit,
        )
        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.edit = async_to_raw_response_wrapper(
            settings.edit,
        )
        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.edit = to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.edit = async_to_streamed_response_wrapper(
            settings.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.intel.sinkhole import Sinkhole

__all__ = ["SinkholesResource", "AsyncSinkholesResource"]


class SinkholesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SinkholesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SinkholesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SinkholesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SinkholesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Sinkhole]:
        """
        List sinkholes owned by this account

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            page=SyncSinglePage[Sinkhole],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Sinkhole,
        )


class AsyncSinkholesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSinkholesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSinkholesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSinkholesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSinkholesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Sinkhole, AsyncSinglePage[Sinkhole]]:
        """
        List sinkholes owned by this account

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/intel/sinkholes", account_id=account_id),
            page=AsyncSinglePage[Sinkhole],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Sinkhole,
        )


class SinkholesResourceWithRawResponse:
    def __init__(self, sinkholes: SinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.list = to_raw_response_wrapper(
            sinkholes.list,
        )


class AsyncSinkholesResourceWithRawResponse:
    def __init__(self, sinkholes: AsyncSinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.list = async_to_raw_response_wrapper(
            sinkholes.list,
        )


class SinkholesResourceWithStreamingResponse:
    def __init__(self, sinkholes: SinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.list = to_streamed_response_wrapper(
            sinkholes.list,
        )


class AsyncSinkholesResourceWithStreamingResponse:
    def __init__(self, sinkholes: AsyncSinkholesResource) -> None:
        self._sinkholes = sinkholes

        self.list = async_to_streamed_response_wrapper(
            sinkholes.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .nel import NEL as NEL
from .ssl import SSL as SSL
from .waf import WAF as WAF
from .ipv6 import IPV6 as IPV6
from .type import Type as Type
from .webp import WebP as WebP
from .zone import Zone as Zone
from .http2 import HTTP2 as HTTP2
from .http3 import HTTP3 as HTTP3
from .brotli import Brotli as Brotli
from .mirage import Mirage as Mirage
from .polish import Polish as Polish
from .ciphers import Ciphers as Ciphers
from .tls_1_3 import TLS1_3 as TLS1_3
from .zero_rtt import ZeroRTT as ZeroRTT
from .ssl_param import SSLParam as SSLParam
from .waf_param import WAFParam as WAFParam
from .websocket import Websocket as Websocket
from .zone_hold import ZoneHold as ZoneHold
from .cache_level import CacheLevel as CacheLevel
from .early_hints import EarlyHints as EarlyHints
from .pseudo_ipv4 import PseudoIPV4 as PseudoIPV4
from .mirage_param import MirageParam as MirageParam
from .polish_param import PolishParam as PolishParam
from .advanced_ddos import AdvancedDDoS as AdvancedDDoS
from .always_online import AlwaysOnline as AlwaysOnline
from .browser_check import BrowserCheck as BrowserCheck
from .challenge_ttl import ChallengeTTL as ChallengeTTL
from .rocket_loader import RocketLoader as RocketLoader
from .image_resizing import ImageResizing as ImageResizing
from .ip_geolocation import IPGeolocation as IPGeolocation
from .security_level import SecurityLevel as SecurityLevel
from .min_tls_version import MinTLSVersion as MinTLSVersion
from .ssl_recommender import SSLRecommender as SSLRecommender
from .tls_client_auth import TLSClientAuth as TLSClientAuth
from .always_use_https import AlwaysUseHTTPS as AlwaysUseHTTPS
from .development_mode import DevelopmentMode as DevelopmentMode
from .hold_edit_params import HoldEditParams as HoldEditParams
from .orange_to_orange import OrangeToOrange as OrangeToOrange
from .prefetch_preload import PrefetchPreload as PrefetchPreload
from .security_headers import SecurityHeaders as SecurityHeaders
from .zone_edit_params import ZoneEditParams as ZoneEditParams
from .zone_list_params import ZoneListParams as ZoneListParams
from .browser_cache_ttl import BrowserCacheTTL as BrowserCacheTTL
from .cache_level_param import CacheLevelParam as CacheLevelParam
from .email_obfuscation import EmailObfuscation as EmailObfuscation
from .h2_prioritization import H2Prioritization as H2Prioritization
from .hold_create_params import HoldCreateParams as HoldCreateParams
from .hold_delete_params import HoldDeleteParams as HoldDeleteParams
from .hotlink_protection import HotlinkProtection as HotlinkProtection
from .proxy_read_timeout import ProxyReadTimeout as ProxyReadTimeout
from .response_buffering import ResponseBuffering as ResponseBuffering
from .zone_create_params import ZoneCreateParams as ZoneCreateParams
from .available_rate_plan import AvailableRatePlan as AvailableRatePlan
from .browser_check_param import BrowserCheckParam as BrowserCheckParam
from .opportunistic_onion import OpportunisticOnion as OpportunisticOnion
from .rocket_loader_param import RocketLoaderParam as RocketLoaderParam
from .setting_edit_params import SettingEditParams as SettingEditParams
from .ip_geolocation_param import IPGeolocationParam as IPGeolocationParam
from .security_level_param import SecurityLevelParam as SecurityLevelParam
from .server_side_excludes import ServerSideExcludes as ServerSideExcludes
from .setting_get_response import SettingGetResponse as SettingGetResponse
from .zone_delete_response import ZoneDeleteResponse as ZoneDeleteResponse
from .setting_edit_response import SettingEditResponse as SettingEditResponse
from .true_client_ip_header import TrueClientIPHeader as TrueClientIPHeader
from .always_use_https_param import AlwaysUseHTTPSParam as AlwaysUseHTTPSParam
from .rate_plan_get_response import RatePlanGetResponse as RatePlanGetResponse
from .browser_cache_ttl_param import BrowserCacheTTLParam as BrowserCacheTTLParam
from .email_obfuscation_param import EmailObfuscationParam as EmailObfuscationParam
from .environment_edit_params import EnvironmentEditParams as EnvironmentEditParams
from .automatic_https_rewrites import AutomaticHTTPSRewrites as AutomaticHTTPSRewrites
from .opportunistic_encryption import OpportunisticEncryption as OpportunisticEncryption
from .response_buffering_param import ResponseBufferingParam as ResponseBufferingParam
from .environment_create_params import EnvironmentCreateParams as EnvironmentCreateParams
from .environment_edit_response import EnvironmentEditResponse as EnvironmentEditResponse
from .environment_list_response import EnvironmentListResponse as EnvironmentListResponse
from .environment_update_params import EnvironmentUpdateParams as EnvironmentUpdateParams
from .subscription_get_response import SubscriptionGetResponse as SubscriptionGetResponse
from .subscription_create_params import SubscriptionCreateParams as SubscriptionCreateParams
from .subscription_update_params import SubscriptionUpdateParams as SubscriptionUpdateParams
from .environment_create_response import EnvironmentCreateResponse as EnvironmentCreateResponse
from .environment_delete_response import EnvironmentDeleteResponse as EnvironmentDeleteResponse
from .environment_update_response import EnvironmentUpdateResponse as EnvironmentUpdateResponse
from .origin_error_page_pass_thru import OriginErrorPagePassThru as OriginErrorPagePassThru
from .sort_query_string_for_cache import SortQueryStringForCache as SortQueryStringForCache
from .true_client_ip_header_param import TrueClientIPHeaderParam as TrueClientIPHeaderParam
from .subscription_create_response import SubscriptionCreateResponse as SubscriptionCreateResponse
from .subscription_update_response import SubscriptionUpdateResponse as SubscriptionUpdateResponse
from .environment_rollback_response import EnvironmentRollbackResponse as EnvironmentRollbackResponse
from .automatic_https_rewrites_param import AutomaticHTTPSRewritesParam as AutomaticHTTPSRewritesParam
from .custom_nameserver_get_response import CustomNameserverGetResponse as CustomNameserverGetResponse
from .opportunistic_encryption_param import OpportunisticEncryptionParam as OpportunisticEncryptionParam
from .automatic_platform_optimization import AutomaticPlatformOptimization as AutomaticPlatformOptimization
from .custom_nameserver_update_params import CustomNameserverUpdateParams as CustomNameserverUpdateParams
from .activation_check_trigger_response import ActivationCheckTriggerResponse as ActivationCheckTriggerResponse
from .custom_nameserver_update_response import CustomNameserverUpdateResponse as CustomNameserverUpdateResponse
from .origin_error_page_pass_thru_param import OriginErrorPagePassThruParam as OriginErrorPagePassThruParam
from .sort_query_string_for_cache_param import SortQueryStringForCacheParam as SortQueryStringForCacheParam
from .automatic_platform_optimization_param import (
    AutomaticPlatformOptimizationParam as AutomaticPlatformOptimizationParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ActiveSessionKickParticipantsResponse", "Data", "DataParticipant"]


class DataParticipant(BaseModel):
    id: str
    """ID of the session participant"""

    created_at: str

    updated_at: str

    email: Optional[str] = None
    """Email of the session participant."""

    name: Optional[str] = None
    """Name of the session participant."""

    picture: Optional[str] = None
    """A URL pointing to a picture of the participant."""


class Data(BaseModel):
    action: Optional[str] = None

    participants: Optional[List[DataParticipant]] = None


class ActiveSessionKickParticipantsResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from .assets import (
    AssetsResource,
    AsyncAssetsResource,
    AssetsResourceWithRawResponse,
    AsyncAssetsResourceWithRawResponse,
    AssetsResourceWithStreamingResponse,
    AsyncAssetsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.custom_pages import custom_page_update_params
from ...types.custom_pages.custom_page_get_response import CustomPageGetResponse
from ...types.custom_pages.custom_page_list_response import CustomPageListResponse
from ...types.custom_pages.custom_page_update_response import CustomPageUpdateResponse

__all__ = ["CustomPagesResource", "AsyncCustomPagesResource"]


class CustomPagesResource(SyncAPIResource):
    @cached_property
    def assets(self) -> AssetsResource:
        return AssetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CustomPagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomPagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomPagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomPagesResourceWithStreamingResponse(self)

    def update(
        self,
        identifier: Literal[
            "1000_errors",
            "500_errors",
            "basic_challenge",
            "country_challenge",
            "ip_block",
            "managed_challenge",
            "ratelimit_block",
            "under_attack",
            "waf_block",
            "waf_challenge",
        ],
        *,
        state: Literal["default", "customized"],
        url: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPageUpdateResponse]:
        """
        Updates the configuration of an existing custom page.

        Args:
          identifier: Error Page Types

          state: The custom page state.

          url: The URL associated with the custom page.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages/{identifier}",
                identifier=identifier,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "state": state,
                    "url": url,
                },
                custom_page_update_params.CustomPageUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPageUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPageUpdateResponse]], ResultWrapper[CustomPageUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CustomPageListResponse]:
        """
        Fetches all the custom pages.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[CustomPageListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomPageListResponse,
        )

    def get(
        self,
        identifier: Literal[
            "1000_errors",
            "500_errors",
            "basic_challenge",
            "country_challenge",
            "ip_block",
            "managed_challenge",
            "ratelimit_block",
            "under_attack",
            "waf_block",
            "waf_challenge",
        ],
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPageGetResponse]:
        """
        Fetches the details of a custom page.

        Args:
          identifier: Error Page Types

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages/{identifier}",
                identifier=identifier,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPageGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPageGetResponse]], ResultWrapper[CustomPageGetResponse]),
        )


class AsyncCustomPagesResource(AsyncAPIResource):
    @cached_property
    def assets(self) -> AsyncAssetsResource:
        return AsyncAssetsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCustomPagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomPagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomPagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomPagesResourceWithStreamingResponse(self)

    async def update(
        self,
        identifier: Literal[
            "1000_errors",
            "500_errors",
            "basic_challenge",
            "country_challenge",
            "ip_block",
            "managed_challenge",
            "ratelimit_block",
            "under_attack",
            "waf_block",
            "waf_challenge",
        ],
        *,
        state: Literal["default", "customized"],
        url: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPageUpdateResponse]:
        """
        Updates the configuration of an existing custom page.

        Args:
          identifier: Error Page Types

          state: The custom page state.

          url: The URL associated with the custom page.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._put(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages/{identifier}",
                identifier=identifier,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "state": state,
                    "url": url,
                },
                custom_page_update_params.CustomPageUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPageUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPageUpdateResponse]], ResultWrapper[CustomPageUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomPageListResponse, AsyncSinglePage[CustomPageListResponse]]:
        """
        Fetches all the custom pages.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[CustomPageListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomPageListResponse,
        )

    async def get(
        self,
        identifier: Literal[
            "1000_errors",
            "500_errors",
            "basic_challenge",
            "country_challenge",
            "ip_block",
            "managed_challenge",
            "ratelimit_block",
            "under_attack",
            "waf_block",
            "waf_challenge",
        ],
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPageGetResponse]:
        """
        Fetches the details of a custom page.

        Args:
          identifier: Error Page Types

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/custom_pages/{identifier}",
                identifier=identifier,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPageGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPageGetResponse]], ResultWrapper[CustomPageGetResponse]),
        )


class CustomPagesResourceWithRawResponse:
    def __init__(self, custom_pages: CustomPagesResource) -> None:
        self._custom_pages = custom_pages

        self.update = to_raw_response_wrapper(
            custom_pages.update,
        )
        self.list = to_raw_response_wrapper(
            custom_pages.list,
        )
        self.get = to_raw_response_wrapper(
            custom_pages.get,
        )

    @cached_property
    def assets(self) -> AssetsResourceWithRawResponse:
        return AssetsResourceWithRawResponse(self._custom_pages.assets)


class AsyncCustomPagesResourceWithRawResponse:
    def __init__(self, custom_pages: AsyncCustomPagesResource) -> None:
        self._custom_pages = custom_pages

        self.update = async_to_raw_response_wrapper(
            custom_pages.update,
        )
        self.list = async_to_raw_response_wrapper(
            custom_pages.list,
        )
        self.get = async_to_raw_response_wrapper(
            custom_pages.get,
        )

    @cached_property
    def assets(self) -> AsyncAssetsResourceWithRawResponse:
        return AsyncAssetsResourceWithRawResponse(self._custom_pages.assets)


class CustomPagesResourceWithStreamingResponse:
    def __init__(self, custom_pages: CustomPagesResource) -> None:
        self._custom_pages = custom_pages

        self.update = to_streamed_response_wrapper(
            custom_pages.update,
        )
        self.list = to_streamed_response_wrapper(
            custom_pages.list,
        )
        self.get = to_streamed_response_wrapper(
            custom_pages.get,
        )

    @cached_property
    def assets(self) -> AssetsResourceWithStreamingResponse:
        return AssetsResourceWithStreamingResponse(self._custom_pages.assets)


class AsyncCustomPagesResourceWithStreamingResponse:
    def __init__(self, custom_pages: AsyncCustomPagesResource) -> None:
        self._custom_pages = custom_pages

        self.update = async_to_streamed_response_wrapper(
            custom_pages.update,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_pages.list,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_pages.get,
        )

    @cached_property
    def assets(self) -> AsyncAssetsResourceWithStreamingResponse:
        return AsyncAssetsResourceWithStreamingResponse(self._custom_pages.assets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.rulesets import Phase
from ....types.rulesets.phase import Phase
from ....types.rulesets.phases.version_get_response import VersionGetResponse
from ....types.rulesets.phases.version_list_response import VersionListResponse

__all__ = ["VersionsResource", "AsyncVersionsResource"]


class VersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VersionsResourceWithStreamingResponse(self)

    def list(
        self,
        ruleset_phase: Phase,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[VersionListResponse]:
        """
        Fetches the versions of an account or zone entry point ruleset.

        Args:
          ruleset_phase: The phase of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_phase:
            raise ValueError(f"Expected a non-empty value for `ruleset_phase` but received {ruleset_phase!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/phases/{ruleset_phase}/entrypoint/versions",
                ruleset_phase=ruleset_phase,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=VersionListResponse,
        )

    def get(
        self,
        ruleset_version: str,
        *,
        ruleset_phase: Phase,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Fetches a specific version of an account or zone entry point ruleset.

        Args:
          ruleset_phase: The phase of the ruleset.

          ruleset_version: The version of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_phase:
            raise ValueError(f"Expected a non-empty value for `ruleset_phase` but received {ruleset_phase!r}")
        if not ruleset_version:
            raise ValueError(f"Expected a non-empty value for `ruleset_version` but received {ruleset_version!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/phases/{ruleset_phase}/entrypoint/versions/{ruleset_version}",
                ruleset_phase=ruleset_phase,
                ruleset_version=ruleset_version,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )


class AsyncVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVersionsResourceWithStreamingResponse(self)

    def list(
        self,
        ruleset_phase: Phase,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VersionListResponse, AsyncSinglePage[VersionListResponse]]:
        """
        Fetches the versions of an account or zone entry point ruleset.

        Args:
          ruleset_phase: The phase of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_phase:
            raise ValueError(f"Expected a non-empty value for `ruleset_phase` but received {ruleset_phase!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/phases/{ruleset_phase}/entrypoint/versions",
                ruleset_phase=ruleset_phase,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=VersionListResponse,
        )

    async def get(
        self,
        ruleset_version: str,
        *,
        ruleset_phase: Phase,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Fetches a specific version of an account or zone entry point ruleset.

        Args:
          ruleset_phase: The phase of the ruleset.

          ruleset_version: The version of the ruleset.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not ruleset_phase:
            raise ValueError(f"Expected a non-empty value for `ruleset_phase` but received {ruleset_phase!r}")
        if not ruleset_version:
            raise ValueError(f"Expected a non-empty value for `ruleset_version` but received {ruleset_version!r}")
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._get(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/rulesets/phases/{ruleset_phase}/entrypoint/versions/{ruleset_version}",
                ruleset_phase=ruleset_phase,
                ruleset_version=ruleset_version,
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )


class VersionsResourceWithRawResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.list = to_raw_response_wrapper(
            versions.list,
        )
        self.get = to_raw_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithRawResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.list = async_to_raw_response_wrapper(
            versions.list,
        )
        self.get = async_to_raw_response_wrapper(
            versions.get,
        )


class VersionsResourceWithStreamingResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.list = to_streamed_response_wrapper(
            versions.list,
        )
        self.get = to_streamed_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithStreamingResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.list = async_to_streamed_response_wrapper(
            versions.list,
        )
        self.get = async_to_streamed_response_wrapper(
            versions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ownership import Ownership as Ownership
from .ownership_create_params import OwnershipCreateParams as OwnershipCreateParams
from .ownership_validate_params import OwnershipValidateParams as OwnershipValidateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DomainEditResponse", "Authorization", "EmailsProcessed"]


class Authorization(BaseModel):
    authorized: bool

    timestamp: datetime

    status_message: Optional[str] = None


class EmailsProcessed(BaseModel):
    timestamp: datetime

    total_emails_processed: int

    total_emails_processed_previous: int


class DomainEditResponse(BaseModel):
    id: Optional[str] = None
    """Domain identifier"""

    allowed_delivery_modes: Optional[List[Literal["DIRECT", "BCC", "JOURNAL", "API", "RETRO_SCAN"]]] = None

    authorization: Optional[Authorization] = None

    created_at: Optional[datetime] = None

    dmarc_status: Optional[Literal["none", "good", "invalid"]] = None

    domain: Optional[str] = None

    drop_dispositions: Optional[
        List[
            Literal[
                "MALICIOUS",
                "MALICIOUS-BEC",
                "SUSPICIOUS",
                "SPOOF",
                "SPAM",
                "BULK",
                "ENCRYPTED",
                "EXTERNAL",
                "UNKNOWN",
                "NONE",
            ]
        ]
    ] = None

    emails_processed: Optional[EmailsProcessed] = None

    folder: Optional[Literal["AllItems", "Inbox"]] = None

    inbox_provider: Optional[Literal["Microsoft", "Google"]] = None

    integration_id: Optional[str] = None

    ip_restrictions: Optional[List[str]] = None

    last_modified: Optional[datetime] = None
    """Deprecated, use `modified_at` instead. End of life: November 1, 2026."""

    lookback_hops: Optional[int] = None

    modified_at: Optional[datetime] = None

    o365_tenant_id: Optional[str] = None

    regions: Optional[List[Literal["GLOBAL", "AU", "DE", "IN", "US"]]] = None

    require_tls_inbound: Optional[bool] = None

    require_tls_outbound: Optional[bool] = None

    spf_status: Optional[Literal["none", "good", "neutral", "open", "invalid"]] = None

    status: Optional[Literal["pending", "active", "failed", "timeout"]] = None

    transport: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.logpush import validate_origin_params, validate_destination_params, validate_destination_exists_params
from ...types.logpush.validate_origin_response import ValidateOriginResponse
from ...types.logpush.validate_destination_response import ValidateDestinationResponse
from ...types.logpush.validate_destination_exists_response import ValidateDestinationExistsResponse

__all__ = ["ValidateResource", "AsyncValidateResource"]


class ValidateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ValidateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ValidateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ValidateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ValidateResourceWithStreamingResponse(self)

    def destination(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateDestinationResponse]:
        """
        Validates destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/destination",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {"destination_conf": destination_conf}, validate_destination_params.ValidateDestinationParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateDestinationResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ValidateDestinationResponse]], ResultWrapper[ValidateDestinationResponse]),
        )

    def destination_exists(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateDestinationExistsResponse]:
        """
        Checks if there is an existing job with a destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/destination/exists",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {"destination_conf": destination_conf},
                validate_destination_exists_params.ValidateDestinationExistsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateDestinationExistsResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ValidateDestinationExistsResponse]], ResultWrapper[ValidateDestinationExistsResponse]
            ),
        )

    def origin(
        self,
        *,
        logpull_options: Optional[str],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateOriginResponse]:
        """
        Validates logpull origin with logpull_options.

        Args:
          logpull_options: This field is deprecated. Use `output_options` instead. Configuration string. It
              specifies things like requested fields and timestamp formats. If migrating from
              the logpull api, copy the url (full url or just the query string) of your call
              here, and logpush will keep on making this call for you, setting start and end
              times appropriately.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/origin",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform({"logpull_options": logpull_options}, validate_origin_params.ValidateOriginParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateOriginResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ValidateOriginResponse]], ResultWrapper[ValidateOriginResponse]),
        )


class AsyncValidateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncValidateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncValidateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncValidateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncValidateResourceWithStreamingResponse(self)

    async def destination(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateDestinationResponse]:
        """
        Validates destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/destination",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {"destination_conf": destination_conf}, validate_destination_params.ValidateDestinationParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateDestinationResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ValidateDestinationResponse]], ResultWrapper[ValidateDestinationResponse]),
        )

    async def destination_exists(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateDestinationExistsResponse]:
        """
        Checks if there is an existing job with a destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/destination/exists",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {"destination_conf": destination_conf},
                validate_destination_exists_params.ValidateDestinationExistsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateDestinationExistsResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[ValidateDestinationExistsResponse]], ResultWrapper[ValidateDestinationExistsResponse]
            ),
        )

    async def origin(
        self,
        *,
        logpull_options: Optional[str],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ValidateOriginResponse]:
        """
        Validates logpull origin with logpull_options.

        Args:
          logpull_options: This field is deprecated. Use `output_options` instead. Configuration string. It
              specifies things like requested fields and timestamp formats. If migrating from
              the logpull api, copy the url (full url or just the query string) of your call
              here, and logpush will keep on making this call for you, setting start and end
              times appropriately.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/validate/origin",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {"logpull_options": logpull_options}, validate_origin_params.ValidateOriginParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ValidateOriginResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ValidateOriginResponse]], ResultWrapper[ValidateOriginResponse]),
        )


class ValidateResourceWithRawResponse:
    def __init__(self, validate: ValidateResource) -> None:
        self._validate = validate

        self.destination = to_raw_response_wrapper(
            validate.destination,
        )
        self.destination_exists = to_raw_response_wrapper(
            validate.destination_exists,
        )
        self.origin = to_raw_response_wrapper(
            validate.origin,
        )


class AsyncValidateResourceWithRawResponse:
    def __init__(self, validate: AsyncValidateResource) -> None:
        self._validate = validate

        self.destination = async_to_raw_response_wrapper(
            validate.destination,
        )
        self.destination_exists = async_to_raw_response_wrapper(
            validate.destination_exists,
        )
        self.origin = async_to_raw_response_wrapper(
            validate.origin,
        )


class ValidateResourceWithStreamingResponse:
    def __init__(self, validate: ValidateResource) -> None:
        self._validate = validate

        self.destination = to_streamed_response_wrapper(
            validate.destination,
        )
        self.destination_exists = to_streamed_response_wrapper(
            validate.destination_exists,
        )
        self.origin = to_streamed_response_wrapper(
            validate.origin,
        )


class AsyncValidateResourceWithStreamingResponse:
    def __init__(self, validate: AsyncValidateResource) -> None:
        self._validate = validate

        self.destination = async_to_streamed_response_wrapper(
            validate.destination,
        )
        self.destination_exists = async_to_streamed_response_wrapper(
            validate.destination_exists,
        )
        self.origin = async_to_streamed_response_wrapper(
            validate.origin,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import FileTypes

__all__ = ["BinaryStorageCreateParams"]


class BinaryStorageCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    file: Required[FileTypes]
    """The binary file content to upload."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["BlockSenderEditResponse"]


class BlockSenderEditResponse(BaseModel):
    """A blocked sender pattern"""

    id: Optional[str] = None
    """Blocked sender pattern identifier"""

    comments: Optional[str] = None

    created_at: Optional[datetime] = None

    is_regex: Optional[bool] = None

    last_modified: Optional[datetime] = None
    """Deprecated, use `modified_at` instead. End of life: November 1, 2026."""

    modified_at: Optional[datetime] = None

    pattern: Optional[str] = None

    pattern_type: Optional[Literal["EMAIL", "DOMAIN", "IP", "UNKNOWN"]] = None
    """
    Type of pattern matching. Note: UNKNOWN is deprecated and cannot be used when
    creating or updating policies, but may be returned for existing entries.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ConfigUpdateParams"]


class ConfigUpdateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    enabled: Required[bool]
    """Enables or disables Google Tag Gateway for this zone."""

    endpoint: Required[str]
    """Specifies the endpoint path for proxying Google Tag Manager requests.

    Use an absolute path starting with '/', with no nested paths and alphanumeric
    characters only (e.g. /metrics).
    """

    hide_original_ip: Required[Annotated[bool, PropertyInfo(alias="hideOriginalIp")]]
    """Hides the original client IP address from Google when enabled."""

    measurement_id: Required[Annotated[str, PropertyInfo(alias="measurementId")]]
    """Specify the Google Tag Manager container or measurement ID (e.g.

    GTM-XXXXXXX or G-XXXXXXXXXX).
    """

    set_up_tag: Annotated[Optional[bool], PropertyInfo(alias="setUpTag")]
    """Set up the associated Google Tag on the zone automatically when enabled."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RecordingPauseResumeStopRecordingParams"]


class RecordingPauseResumeStopRecordingParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]

    action: Required[Literal["stop", "pause", "resume"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VersionGetParams"]


class VersionGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    worker_id: Required[str]
    """Identifier for the Worker, which can be ID or name."""

    include: Literal["modules"]
    """
    Whether to include the `modules` property of the version in the response, which
    contains code and sourcemap content and may add several megabytes to the
    response size.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PaginationInfo"]


class PaginationInfo(BaseModel):
    count: Optional[float] = None
    """Total number of results for the requested service"""

    page: Optional[float] = None
    """Current page within paginated list of results"""

    per_page: Optional[float] = None
    """Number of results per page of results"""

    total_count: Optional[float] = None
    """Total results available without any search parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UserEditParams"]


class UserEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    description: str
    """The description of the label"""

    metadata: object
    """Metadata for the label"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["SiteLocationParam"]


class SiteLocationParam(TypedDict, total=False):
    """Location of site in latitude and longitude."""

    lat: str
    """Latitude"""

    lon: str
    """Longitude"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

__all__ = ["RegionGetResponse"]

RegionGetResponse: TypeAlias = Union[Optional[str], Optional[object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CountryConfiguration"]


class CountryConfiguration(BaseModel):
    target: Optional[Literal["country"]] = None
    """The configuration target.

    You must set the target to `country` when specifying a country code in the rule.
    """

    value: Optional[str] = None
    """The two-letter ISO-3166-1 alpha-2 code to match.

    For more information, refer to
    [IP Access rules: Parameters](https://developers.cloudflare.com/waf/tools/ip-access-rules/parameters/#country).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["EmailSendingSendRawResponse"]


class EmailSendingSendRawResponse(BaseModel):
    delivered: List[str]
    """Email addresses to which the message was delivered immediately."""

    permanent_bounces: List[str]
    """Email addresses that permanently bounced."""

    queued: List[str]
    """Email addresses for which delivery was queued for later."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RecipientGetParams"]


class RecipientGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier."""

    share_id: Required[str]
    """Share identifier tag."""

    include_resources: bool
    """Include resources in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .live_stat import LiveStat
from ...._models import BaseModel

__all__ = ["FleetStatusLiveResponse", "DeviceStats"]


class DeviceStats(BaseModel):
    by_colo: Optional[List[LiveStat]] = FieldInfo(alias="byColo", default=None)

    by_mode: Optional[List[LiveStat]] = FieldInfo(alias="byMode", default=None)

    by_platform: Optional[List[LiveStat]] = FieldInfo(alias="byPlatform", default=None)

    by_status: Optional[List[LiveStat]] = FieldInfo(alias="byStatus", default=None)

    by_version: Optional[List[LiveStat]] = FieldInfo(alias="byVersion", default=None)

    unique_devices_total: Optional[float] = FieldInfo(alias="uniqueDevicesTotal", default=None)
    """Number of unique devices"""


class FleetStatusLiveResponse(BaseModel):
    device_stats: Optional[DeviceStats] = FieldInfo(alias="deviceStats", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["EvaluationTypeListParams"]


class EvaluationTypeListParams(TypedDict, total=False):
    account_id: Required[str]

    order_by: str

    order_by_direction: Literal["asc", "desc"]

    page: int

    per_page: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .mitigation_list_params import MitigationListParams as MitigationListParams
from .abuse_report_list_params import AbuseReportListParams as AbuseReportListParams
from .mitigation_list_response import MitigationListResponse as MitigationListResponse
from .mitigation_review_params import MitigationReviewParams as MitigationReviewParams
from .abuse_report_get_response import AbuseReportGetResponse as AbuseReportGetResponse
from .abuse_report_create_params import AbuseReportCreateParams as AbuseReportCreateParams
from .abuse_report_list_response import AbuseReportListResponse as AbuseReportListResponse
from .mitigation_review_response import MitigationReviewResponse as MitigationReviewResponse
from .abuse_report_create_response import AbuseReportCreateResponse as AbuseReportCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["BehaviourUpdateParams", "Behaviors"]


class BehaviourUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    behaviors: Required[Dict[str, Behaviors]]


class Behaviors(TypedDict, total=False):
    enabled: Required[bool]

    risk_level: Required[Literal["low", "medium", "high"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .ases import (
    AsesResource,
    AsyncAsesResource,
    AsesResourceWithRawResponse,
    AsyncAsesResourceWithRawResponse,
    AsesResourceWithStreamingResponse,
    AsyncAsesResourceWithStreamingResponse,
)
from .http import (
    HTTPResource,
    AsyncHTTPResource,
    HTTPResourceWithRawResponse,
    AsyncHTTPResourceWithRawResponse,
    HTTPResourceWithStreamingResponse,
    AsyncHTTPResourceWithStreamingResponse,
)
from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from .timeseries_groups import (
    TimeseriesGroupsResource,
    AsyncTimeseriesGroupsResource,
    TimeseriesGroupsResourceWithRawResponse,
    AsyncTimeseriesGroupsResourceWithRawResponse,
    TimeseriesGroupsResourceWithStreamingResponse,
    AsyncTimeseriesGroupsResourceWithStreamingResponse,
)

__all__ = [
    "LocationsResource",
    "AsyncLocationsResource",
    "LocationsResourceWithRawResponse",
    "AsyncLocationsResourceWithRawResponse",
    "LocationsResourceWithStreamingResponse",
    "AsyncLocationsResourceWithStreamingResponse",
    "AsesResource",
    "AsyncAsesResource",
    "AsesResourceWithRawResponse",
    "AsyncAsesResourceWithRawResponse",
    "AsesResourceWithStreamingResponse",
    "AsyncAsesResourceWithStreamingResponse",
    "SummaryResource",
    "AsyncSummaryResource",
    "SummaryResourceWithRawResponse",
    "AsyncSummaryResourceWithRawResponse",
    "SummaryResourceWithStreamingResponse",
    "AsyncSummaryResourceWithStreamingResponse",
    "TimeseriesGroupsResource",
    "AsyncTimeseriesGroupsResource",
    "TimeseriesGroupsResourceWithRawResponse",
    "AsyncTimeseriesGroupsResourceWithRawResponse",
    "TimeseriesGroupsResourceWithStreamingResponse",
    "AsyncTimeseriesGroupsResourceWithStreamingResponse",
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
    "HTTPResource",
    "AsyncHTTPResource",
    "HTTPResourceWithRawResponse",
    "AsyncHTTPResourceWithRawResponse",
    "HTTPResourceWithStreamingResponse",
    "AsyncHTTPResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AuthorityGetResponse", "CertificateAuthority", "CertificateAuthorityRelated"]


class CertificateAuthorityRelated(BaseModel):
    certificate_record_type: Literal["ROOT_CERTIFICATE", "INTERMEDIATE_CERTIFICATE"] = FieldInfo(
        alias="certificateRecordType"
    )
    """Specifies the type of certificate in the trust chain."""

    name: str
    """The full name of the certificate authority (CA)."""

    revocation_status: Literal["NOT_REVOKED", "REVOKED", "PARENT_CERT_REVOKED"] = FieldInfo(alias="revocationStatus")
    """The current revocation status of a Certificate Authority (CA) certificate."""

    sha256_fingerprint: str = FieldInfo(alias="sha256Fingerprint")
    """The SHA-256 fingerprint of the intermediate certificate."""


class CertificateAuthority(BaseModel):
    apple_status: Literal[
        "INCLUDED", "NOT_YET_INCLUDED", "NOT_INCLUDED", "NOT_BEFORE", "REMOVED", "DISABLED", "BLOCKED"
    ] = FieldInfo(alias="appleStatus")
    """The inclusion status of a Certificate Authority (CA) in the trust store."""

    authority_key_identifier: str = FieldInfo(alias="authorityKeyIdentifier")
    """The authorityKeyIdentifier value extracted from the certificate PEM."""

    certificate_record_type: Literal["ROOT_CERTIFICATE", "INTERMEDIATE_CERTIFICATE"] = FieldInfo(
        alias="certificateRecordType"
    )
    """Specifies the type of certificate in the trust chain."""

    chrome_status: Literal[
        "INCLUDED", "NOT_YET_INCLUDED", "NOT_INCLUDED", "NOT_BEFORE", "REMOVED", "DISABLED", "BLOCKED"
    ] = FieldInfo(alias="chromeStatus")
    """The inclusion status of a Certificate Authority (CA) in the trust store."""

    country: str
    """The two-letter ISO country code where the CA organization is based."""

    country_name: str = FieldInfo(alias="countryName")
    """The full country name corresponding to the country code."""

    microsoft_status: Literal[
        "INCLUDED", "NOT_YET_INCLUDED", "NOT_INCLUDED", "NOT_BEFORE", "REMOVED", "DISABLED", "BLOCKED"
    ] = FieldInfo(alias="microsoftStatus")
    """The inclusion status of a Certificate Authority (CA) in the trust store."""

    mozilla_status: Literal[
        "INCLUDED", "NOT_YET_INCLUDED", "NOT_INCLUDED", "NOT_BEFORE", "REMOVED", "DISABLED", "BLOCKED"
    ] = FieldInfo(alias="mozillaStatus")
    """The inclusion status of a Certificate Authority (CA) in the trust store."""

    name: str
    """The full name of the certificate authority (CA)."""

    owner: str
    """The organization that owns and operates the CA."""

    parent_name: str = FieldInfo(alias="parentName")
    """
    The name of the parent/root certificate authority that issued this intermediate
    certificate.
    """

    parent_sha256_fingerprint: str = FieldInfo(alias="parentSha256Fingerprint")
    """The SHA-256 fingerprint of the parent certificate."""

    related: List[CertificateAuthorityRelated]
    """CAs from the same owner."""

    revocation_status: Literal["NOT_REVOKED", "REVOKED", "PARENT_CERT_REVOKED"] = FieldInfo(alias="revocationStatus")
    """The current revocation status of a Certificate Authority (CA) certificate."""

    sha256_fingerprint: str = FieldInfo(alias="sha256Fingerprint")
    """The SHA-256 fingerprint of the intermediate certificate."""

    subject_key_identifier: str = FieldInfo(alias="subjectKeyIdentifier")
    """The subjectKeyIdentifier value extracted from the certificate PEM."""

    valid_from: str = FieldInfo(alias="validFrom")
    """The start date of the certificate’s validity period (ISO format)."""

    valid_to: str = FieldInfo(alias="validTo")
    """The end date of the certificate’s validity period (ISO format)."""


class AuthorityGetResponse(BaseModel):
    certificate_authority: CertificateAuthority = FieldInfo(alias="certificateAuthority")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .dhcp_relay_param import DHCPRelayParam
from .dhcp_server_param import DHCPServerParam

__all__ = ["LANStaticAddressingParam"]


class LANStaticAddressingParam(TypedDict, total=False):
    """
    If the site is not configured in high availability mode, this configuration is optional (if omitted, use DHCP). However, if in high availability mode, static_address is required along with secondary and virtual address.
    """

    address: Required[str]
    """A valid CIDR notation representing an IP range."""

    dhcp_relay: DHCPRelayParam

    dhcp_server: DHCPServerParam

    secondary_address: str
    """A valid CIDR notation representing an IP range."""

    virtual_address: str
    """A valid CIDR notation representing an IP range."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fraud import (
    FraudResource,
    AsyncFraudResource,
    FraudResourceWithRawResponse,
    AsyncFraudResourceWithRawResponse,
    FraudResourceWithStreamingResponse,
    AsyncFraudResourceWithStreamingResponse,
)

__all__ = [
    "FraudResource",
    "AsyncFraudResource",
    "FraudResourceWithRawResponse",
    "AsyncFraudResourceWithRawResponse",
    "FraudResourceWithStreamingResponse",
    "AsyncFraudResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel

__all__ = ["TestStatOverTime", "Slot"]


class Slot(BaseModel):
    timestamp: str

    value: int


class TestStatOverTime(BaseModel):
    __test__ = False
    slots: List[Slot]

    avg: Optional[int] = None
    """average observed in the time period"""

    max: Optional[int] = None
    """highest observed in the time period"""

    min: Optional[int] = None
    """lowest observed in the time period"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["ASPASnapshotResponse", "ASNInfo", "ASNInfo_13335", "ASPAObject", "Meta"]


class ASNInfo_13335(BaseModel):
    asn: int
    """ASN number."""

    country: str
    """Alpha-2 country code."""

    name: str
    """AS name."""


class ASNInfo(BaseModel):
    api_13335: ASNInfo_13335 = FieldInfo(alias="13335")


class ASPAObject(BaseModel):
    customer_asn: int = FieldInfo(alias="customerAsn")
    """The customer ASN publishing the ASPA object."""

    providers: List[int]


class Meta(BaseModel):
    data_time: datetime = FieldInfo(alias="dataTime")
    """Timestamp of the underlying data."""

    query_time: datetime = FieldInfo(alias="queryTime")
    """Timestamp when the query was executed."""

    total_count: int = FieldInfo(alias="totalCount")
    """Total number of ASPA objects."""


class ASPASnapshotResponse(BaseModel):
    asn_info: ASNInfo = FieldInfo(alias="asnInfo")

    aspa_objects: List[ASPAObject] = FieldInfo(alias="aspaObjects")

    meta: Meta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .job_logs_params import JobLogsParams as JobLogsParams
from .job_get_response import JobGetResponse as JobGetResponse
from .job_create_params import JobCreateParams as JobCreateParams
from .job_list_response import JobListResponse as JobListResponse
from .job_logs_response import JobLogsResponse as JobLogsResponse
from .job_create_response import JobCreateResponse as JobCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .crawl_get_params import CrawlGetParams as CrawlGetParams
from .pdf_create_params import PDFCreateParams as PDFCreateParams
from .crawl_get_response import CrawlGetResponse as CrawlGetResponse
from .json_create_params import JsonCreateParams as JsonCreateParams
from .link_create_params import LinkCreateParams as LinkCreateParams
from .crawl_create_params import CrawlCreateParams as CrawlCreateParams
from .json_create_response import JsonCreateResponse as JsonCreateResponse
from .link_create_response import LinkCreateResponse as LinkCreateResponse
from .scrape_create_params import ScrapeCreateParams as ScrapeCreateParams
from .content_create_params import ContentCreateParams as ContentCreateParams
from .crawl_create_response import CrawlCreateResponse as CrawlCreateResponse
from .crawl_delete_response import CrawlDeleteResponse as CrawlDeleteResponse
from .markdown_create_params import MarkdownCreateParams as MarkdownCreateParams
from .scrape_create_response import ScrapeCreateResponse as ScrapeCreateResponse
from .snapshot_create_params import SnapshotCreateParams as SnapshotCreateParams
from .content_create_response import ContentCreateResponse as ContentCreateResponse
from .markdown_create_response import MarkdownCreateResponse as MarkdownCreateResponse
from .screenshot_create_params import ScreenshotCreateParams as ScreenshotCreateParams
from .snapshot_create_response import SnapshotCreateResponse as SnapshotCreateResponse
from .screenshot_create_response import ScreenshotCreateResponse as ScreenshotCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .job_list_params import JobListParams as JobListParams
from .job_logs_params import JobLogsParams as JobLogsParams
from .item_list_params import ItemListParams as ItemListParams
from .item_logs_params import ItemLogsParams as ItemLogsParams
from .item_sync_params import ItemSyncParams as ItemSyncParams
from .job_get_response import JobGetResponse as JobGetResponse
from .item_get_response import ItemGetResponse as ItemGetResponse
from .job_create_params import JobCreateParams as JobCreateParams
from .job_list_response import JobListResponse as JobListResponse
from .job_logs_response import JobLogsResponse as JobLogsResponse
from .job_update_params import JobUpdateParams as JobUpdateParams
from .item_chunks_params import ItemChunksParams as ItemChunksParams
from .item_list_response import ItemListResponse as ItemListResponse
from .item_logs_response import ItemLogsResponse as ItemLogsResponse
from .item_sync_response import ItemSyncResponse as ItemSyncResponse
from .item_upload_params import ItemUploadParams as ItemUploadParams
from .job_create_response import JobCreateResponse as JobCreateResponse
from .job_update_response import JobUpdateResponse as JobUpdateResponse
from .item_chunks_response import ItemChunksResponse as ItemChunksResponse
from .item_delete_response import ItemDeleteResponse as ItemDeleteResponse
from .item_upload_response import ItemUploadResponse as ItemUploadResponse
from .item_create_or_update_params import ItemCreateOrUpdateParams as ItemCreateOrUpdateParams
from .item_create_or_update_response import ItemCreateOrUpdateResponse as ItemCreateOrUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["UARuleCreateResponse", "Configuration"]


class Configuration(BaseModel):
    """The configuration object for the current rule."""

    target: Optional[str] = None
    """The configuration target for this rule.

    You must set the target to `ua` for User Agent Blocking rules.
    """

    value: Optional[str] = None
    """The exact user agent string to match.

    This value will be compared to the received `User-Agent` HTTP header value.
    """


class UARuleCreateResponse(BaseModel):
    id: Optional[str] = None
    """The unique identifier of the User Agent Blocking rule."""

    configuration: Optional[Configuration] = None
    """The configuration object for the current rule."""

    description: Optional[str] = None
    """An informative summary of the rule."""

    mode: Optional[Literal["block", "challenge", "js_challenge", "managed_challenge"]] = None
    """The action to apply to a matched request."""

    paused: Optional[bool] = None
    """When true, indicates that the rule is currently paused."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["SecurityLevelParam"]


class SecurityLevelParam(TypedDict, total=False):
    id: Literal["security_level"]
    """Control options for the **Security Level** feature from the **Security** app."""

    value: Literal["off", "essentially_off", "low", "medium", "high", "under_attack"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ip_update_params import IPUpdateParams as IPUpdateParams
from .ip_delete_response import IPDeleteResponse as IPDeleteResponse
from .ip_update_response import IPUpdateResponse as IPUpdateResponse
from .zone_update_params import ZoneUpdateParams as ZoneUpdateParams
from .zone_delete_response import ZoneDeleteResponse as ZoneDeleteResponse
from .zone_update_response import ZoneUpdateResponse as ZoneUpdateResponse
from .account_update_params import AccountUpdateParams as AccountUpdateParams
from .account_delete_response import AccountDeleteResponse as AccountDeleteResponse
from .account_update_response import AccountUpdateResponse as AccountUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo
from .issue_type import IssueType
from .severity_query_param import SeverityQueryParam

__all__ = ["IssueListParams"]


class IssueListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    dismissed: bool

    issue_class: SequenceNotStr[str]

    issue_class_neq: Annotated[SequenceNotStr[str], PropertyInfo(alias="issue_class~neq")]

    issue_type: List[IssueType]

    issue_type_neq: Annotated[List[IssueType], PropertyInfo(alias="issue_type~neq")]

    page: int
    """Specifies the current page within paginated list of results."""

    per_page: int
    """Sets the number of results per page of results."""

    product: SequenceNotStr[str]

    product_neq: Annotated[SequenceNotStr[str], PropertyInfo(alias="product~neq")]

    severity: List[SeverityQueryParam]

    severity_neq: Annotated[List[SeverityQueryParam], PropertyInfo(alias="severity~neq")]

    subject: SequenceNotStr[str]

    subject_neq: Annotated[SequenceNotStr[str], PropertyInfo(alias="subject~neq")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ConnectivityPrecheckSourceResponse"]


class ConnectivityPrecheckSourceResponse(BaseModel):
    connectivity_status: Optional[Literal["success", "error"]] = FieldInfo(alias="connectivityStatus", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["IndexInfoResponse"]


class IndexInfoResponse(BaseModel):
    dimensions: Optional[int] = None
    """Specifies the number of dimensions for the index"""

    processed_up_to_datetime: Optional[datetime] = FieldInfo(alias="processedUpToDatetime", default=None)
    """
    Specifies the timestamp the last mutation batch was processed as an ISO8601
    string.
    """

    processed_up_to_mutation: Optional[str] = FieldInfo(alias="processedUpToMutation", default=None)
    """
    The unique identifier for the async mutation operation containing the changeset.
    """

    vector_count: Optional[int] = FieldInfo(alias="vectorCount", default=None)
    """Specifies the number of vectors present in the index"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .stage import Stage as Stage
from .project import Project as Project
from .deployment import Deployment as Deployment
from .project_edit_params import ProjectEditParams as ProjectEditParams
from .project_list_params import ProjectListParams as ProjectListParams
from .project_create_params import ProjectCreateParams as ProjectCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SecurityHeaders", "Value", "ValueStrictTransportSecurity"]


class ValueStrictTransportSecurity(BaseModel):
    """Strict Transport Security."""

    enabled: Optional[bool] = None
    """Whether or not strict transport security is enabled."""

    include_subdomains: Optional[bool] = None
    """Include all subdomains for strict transport security."""

    max_age: Optional[float] = None
    """Max age in seconds of the strict transport security."""

    nosniff: Optional[bool] = None
    """Whether or not to include 'X-Content-Type-Options: nosniff' header."""

    preload: Optional[bool] = None
    """Enable automatic preload of the HSTS configuration."""


class Value(BaseModel):
    """Current value of the zone setting."""

    strict_transport_security: Optional[ValueStrictTransportSecurity] = None
    """Strict Transport Security."""


class SecurityHeaders(BaseModel):
    """Cloudflare security header for a zone."""

    id: Literal["security_header"]
    """ID of the zone's security header."""

    value: Value
    """Current value of the zone setting."""

    editable: Optional[Literal[True, False]] = None
    """
    Whether or not this setting can be modified for this zone (based on your
    Cloudflare plan level).
    """

    modified_on: Optional[datetime] = None
    """last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import TypeAlias

from ....._models import BaseModel

__all__ = ["ItemChunksResponse", "ItemChunksResponseItem", "ItemChunksResponseItemItem"]


class ItemChunksResponseItemItem(BaseModel):
    key: str

    metadata: Optional[Dict[str, object]] = None

    timestamp: Optional[float] = None


class ItemChunksResponseItem(BaseModel):
    id: str

    item: ItemChunksResponseItemItem

    text: str

    end_byte: Optional[float] = None

    start_byte: Optional[float] = None


ItemChunksResponse: TypeAlias = List[ItemChunksResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .permission_grant import PermissionGrant

__all__ = ["Role", "Permissions"]


class Permissions(BaseModel):
    analytics: Optional[PermissionGrant] = None

    billing: Optional[PermissionGrant] = None

    cache_purge: Optional[PermissionGrant] = None

    dns: Optional[PermissionGrant] = None

    dns_records: Optional[PermissionGrant] = None

    lb: Optional[PermissionGrant] = None

    logs: Optional[PermissionGrant] = None

    organization: Optional[PermissionGrant] = None

    ssl: Optional[PermissionGrant] = None

    waf: Optional[PermissionGrant] = None

    zone_settings: Optional[PermissionGrant] = None

    zones: Optional[PermissionGrant] = None


class Role(BaseModel):
    id: str
    """Role identifier tag."""

    description: str
    """Description of role's permissions."""

    name: str
    """Role name."""

    permissions: Permissions



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .type_get_params import TypeGetParams as TypeGetParams
from .class_get_params import ClassGetParams as ClassGetParams
from .type_get_response import TypeGetResponse as TypeGetResponse
from .class_get_response import ClassGetResponse as ClassGetResponse
from .severity_get_params import SeverityGetParams as SeverityGetParams
from .context_get_response import ContextGetResponse as ContextGetResponse
from .audit_log_list_params import AuditLogListParams as AuditLogListParams
from .severity_get_response import SeverityGetResponse as SeverityGetResponse
from .audit_log_list_response import AuditLogListResponse as AuditLogListResponse
from .classification_update_params import ClassificationUpdateParams as ClassificationUpdateParams
from .classification_update_response import ClassificationUpdateResponse as ClassificationUpdateResponse
from .audit_log_list_by_insight_params import AuditLogListByInsightParams as AuditLogListByInsightParams
from .audit_log_list_by_insight_response import AuditLogListByInsightResponse as AuditLogListByInsightResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.network_interconnects import interconnect_list_params, interconnect_create_params
from ...types.network_interconnects.interconnect_get_response import InterconnectGetResponse
from ...types.network_interconnects.interconnect_list_response import InterconnectListResponse
from ...types.network_interconnects.interconnect_create_response import InterconnectCreateResponse
from ...types.network_interconnects.interconnect_status_response import InterconnectStatusResponse

__all__ = ["InterconnectsResource", "AsyncInterconnectsResource"]


class InterconnectsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InterconnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return InterconnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InterconnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return InterconnectsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        account: str,
        slot_id: str,
        type: str,
        speed: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        """
        Create a new interconnect

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        account: str,
        bandwidth: Literal["50M", "100M", "200M", "300M", "400M", "500M", "1G", "2G", "5G", "10G", "20G", "50G"],
        pairing_key: str,
        type: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        """
        Create a new interconnect

        Args:
          account_id: Customer account tag

          bandwidth: Bandwidth structure as visible through the customer-facing API.

          pairing_key: Pairing key provided by GCP

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "account", "slot_id", "type"], ["account_id", "account", "bandwidth", "pairing_key", "type"]
    )
    def create(
        self,
        *,
        account_id: str,
        account: str,
        slot_id: str | Omit = omit,
        type: str,
        speed: Optional[str] | Omit = omit,
        bandwidth: Literal["50M", "100M", "200M", "300M", "400M", "500M", "1G", "2G", "5G", "10G", "20G", "50G"]
        | Omit = omit,
        pairing_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            InterconnectCreateResponse,
            self._post(
                path_template("/accounts/{account_id}/cni/interconnects", account_id=account_id),
                body=maybe_transform(
                    {
                        "account": account,
                        "slot_id": slot_id,
                        "type": type,
                        "speed": speed,
                        "bandwidth": bandwidth,
                        "pairing_key": pairing_key,
                    },
                    interconnect_create_params.InterconnectCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        cursor: Optional[int] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        site: Optional[str] | Omit = omit,
        type: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectListResponse:
        """
        List existing interconnects

        Args:
          account_id: Customer account tag

          site: If specified, only show interconnects located at the given site

          type: If specified, only show interconnects of the given type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cni/interconnects", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "site": site,
                        "type": type,
                    },
                    interconnect_list_params.InterconnectListParams,
                ),
            ),
            cast_to=InterconnectListResponse,
        )

    def delete(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/accounts/{account_id}/cni/interconnects/{icon}", account_id=account_id, icon=icon),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectGetResponse:
        """
        Get information about an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        return cast(
            InterconnectGetResponse,
            self._get(
                path_template("/accounts/{account_id}/cni/interconnects/{icon}", account_id=account_id, icon=icon),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def loa(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Generate the Letter of Authorization (LOA) for a given interconnect

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template("/accounts/{account_id}/cni/interconnects/{icon}/loa", account_id=account_id, icon=icon),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def status(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectStatusResponse:
        """
        Get the current status of an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        return cast(
            InterconnectStatusResponse,
            self._get(
                path_template(
                    "/accounts/{account_id}/cni/interconnects/{icon}/status", account_id=account_id, icon=icon
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectStatusResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncInterconnectsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInterconnectsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInterconnectsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInterconnectsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncInterconnectsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        account: str,
        slot_id: str,
        type: str,
        speed: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        """
        Create a new interconnect

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        account: str,
        bandwidth: Literal["50M", "100M", "200M", "300M", "400M", "500M", "1G", "2G", "5G", "10G", "20G", "50G"],
        pairing_key: str,
        type: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        """
        Create a new interconnect

        Args:
          account_id: Customer account tag

          bandwidth: Bandwidth structure as visible through the customer-facing API.

          pairing_key: Pairing key provided by GCP

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "account", "slot_id", "type"], ["account_id", "account", "bandwidth", "pairing_key", "type"]
    )
    async def create(
        self,
        *,
        account_id: str,
        account: str,
        slot_id: str | Omit = omit,
        type: str,
        speed: Optional[str] | Omit = omit,
        bandwidth: Literal["50M", "100M", "200M", "300M", "400M", "500M", "1G", "2G", "5G", "10G", "20G", "50G"]
        | Omit = omit,
        pairing_key: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectCreateResponse:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            InterconnectCreateResponse,
            await self._post(
                path_template("/accounts/{account_id}/cni/interconnects", account_id=account_id),
                body=await async_maybe_transform(
                    {
                        "account": account,
                        "slot_id": slot_id,
                        "type": type,
                        "speed": speed,
                        "bandwidth": bandwidth,
                        "pairing_key": pairing_key,
                    },
                    interconnect_create_params.InterconnectCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectCreateResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        account_id: str,
        cursor: Optional[int] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        site: Optional[str] | Omit = omit,
        type: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectListResponse:
        """
        List existing interconnects

        Args:
          account_id: Customer account tag

          site: If specified, only show interconnects located at the given site

          type: If specified, only show interconnects of the given type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cni/interconnects", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "site": site,
                        "type": type,
                    },
                    interconnect_list_params.InterconnectListParams,
                ),
            ),
            cast_to=InterconnectListResponse,
        )

    async def delete(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/accounts/{account_id}/cni/interconnects/{icon}", account_id=account_id, icon=icon),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectGetResponse:
        """
        Get information about an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        return cast(
            InterconnectGetResponse,
            await self._get(
                path_template("/accounts/{account_id}/cni/interconnects/{icon}", account_id=account_id, icon=icon),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def loa(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Generate the Letter of Authorization (LOA) for a given interconnect

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template("/accounts/{account_id}/cni/interconnects/{icon}/loa", account_id=account_id, icon=icon),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def status(
        self,
        icon: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InterconnectStatusResponse:
        """
        Get the current status of an interconnect object

        Args:
          account_id: Customer account tag

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not icon:
            raise ValueError(f"Expected a non-empty value for `icon` but received {icon!r}")
        return cast(
            InterconnectStatusResponse,
            await self._get(
                path_template(
                    "/accounts/{account_id}/cni/interconnects/{icon}/status", account_id=account_id, icon=icon
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, InterconnectStatusResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class InterconnectsResourceWithRawResponse:
    def __init__(self, interconnects: InterconnectsResource) -> None:
        self._interconnects = interconnects

        self.create = to_raw_response_wrapper(
            interconnects.create,
        )
        self.list = to_raw_response_wrapper(
            interconnects.list,
        )
        self.delete = to_raw_response_wrapper(
            interconnects.delete,
        )
        self.get = to_raw_response_wrapper(
            interconnects.get,
        )
        self.loa = to_raw_response_wrapper(
            interconnects.loa,
        )
        self.status = to_raw_response_wrapper(
            interconnects.status,
        )


class AsyncInterconnectsResourceWithRawResponse:
    def __init__(self, interconnects: AsyncInterconnectsResource) -> None:
        self._interconnects = interconnects

        self.create = async_to_raw_response_wrapper(
            interconnects.create,
        )
        self.list = async_to_raw_response_wrapper(
            interconnects.list,
        )
        self.delete = async_to_raw_response_wrapper(
            interconnects.delete,
        )
        self.get = async_to_raw_response_wrapper(
            interconnects.get,
        )
        self.loa = async_to_raw_response_wrapper(
            interconnects.loa,
        )
        self.status = async_to_raw_response_wrapper(
            interconnects.status,
        )


class InterconnectsResourceWithStreamingResponse:
    def __init__(self, interconnects: InterconnectsResource) -> None:
        self._interconnects = interconnects

        self.create = to_streamed_response_wrapper(
            interconnects.create,
        )
        self.list = to_streamed_response_wrapper(
            interconnects.list,
        )
        self.delete = to_streamed_response_wrapper(
            interconnects.delete,
        )
        self.get = to_streamed_response_wrapper(
            interconnects.get,
        )
        self.loa = to_streamed_response_wrapper(
            interconnects.loa,
        )
        self.status = to_streamed_response_wrapper(
            interconnects.status,
        )


class AsyncInterconnectsResourceWithStreamingResponse:
    def __init__(self, interconnects: AsyncInterconnectsResource) -> None:
        self._interconnects = interconnects

        self.create = async_to_streamed_response_wrapper(
            interconnects.create,
        )
        self.list = async_to_streamed_response_wrapper(
            interconnects.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            interconnects.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            interconnects.get,
        )
        self.loa = async_to_streamed_response_wrapper(
            interconnects.loa,
        )
        self.status = async_to_streamed_response_wrapper(
            interconnects.status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .os import (
    OSResource,
    AsyncOSResource,
    OSResourceWithRawResponse,
    AsyncOSResourceWithRawResponse,
    OSResourceWithStreamingResponse,
    AsyncOSResourceWithStreamingResponse,
)
from .bot_class import (
    BotClassResource,
    AsyncBotClassResource,
    BotClassResourceWithRawResponse,
    AsyncBotClassResourceWithRawResponse,
    BotClassResourceWithStreamingResponse,
    AsyncBotClassResourceWithStreamingResponse,
)
from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from .ip_version import (
    IPVersionResource,
    AsyncIPVersionResource,
    IPVersionResourceWithRawResponse,
    AsyncIPVersionResourceWithRawResponse,
    IPVersionResourceWithStreamingResponse,
    AsyncIPVersionResourceWithStreamingResponse,
)
from .device_type import (
    DeviceTypeResource,
    AsyncDeviceTypeResource,
    DeviceTypeResourceWithRawResponse,
    AsyncDeviceTypeResourceWithRawResponse,
    DeviceTypeResourceWithStreamingResponse,
    AsyncDeviceTypeResourceWithStreamingResponse,
)
from .http_method import (
    HTTPMethodResource,
    AsyncHTTPMethodResource,
    HTTPMethodResourceWithRawResponse,
    AsyncHTTPMethodResourceWithRawResponse,
    HTTPMethodResourceWithStreamingResponse,
    AsyncHTTPMethodResourceWithStreamingResponse,
)
from .tls_version import (
    TLSVersionResource,
    AsyncTLSVersionResource,
    TLSVersionResourceWithRawResponse,
    AsyncTLSVersionResourceWithRawResponse,
    TLSVersionResourceWithStreamingResponse,
    AsyncTLSVersionResourceWithStreamingResponse,
)
from .http_protocol import (
    HTTPProtocolResource,
    AsyncHTTPProtocolResource,
    HTTPProtocolResourceWithRawResponse,
    AsyncHTTPProtocolResourceWithRawResponse,
    HTTPProtocolResourceWithStreamingResponse,
    AsyncHTTPProtocolResourceWithStreamingResponse,
)
from .browser_family import (
    BrowserFamilyResource,
    AsyncBrowserFamilyResource,
    BrowserFamilyResourceWithRawResponse,
    AsyncBrowserFamilyResourceWithRawResponse,
    BrowserFamilyResourceWithStreamingResponse,
    AsyncBrowserFamilyResourceWithStreamingResponse,
)

__all__ = [
    "BotClassResource",
    "AsyncBotClassResource",
    "BotClassResourceWithRawResponse",
    "AsyncBotClassResourceWithRawResponse",
    "BotClassResourceWithStreamingResponse",
    "AsyncBotClassResourceWithStreamingResponse",
    "DeviceTypeResource",
    "AsyncDeviceTypeResource",
    "DeviceTypeResourceWithRawResponse",
    "AsyncDeviceTypeResourceWithRawResponse",
    "DeviceTypeResourceWithStreamingResponse",
    "AsyncDeviceTypeResourceWithStreamingResponse",
    "HTTPProtocolResource",
    "AsyncHTTPProtocolResource",
    "HTTPProtocolResourceWithRawResponse",
    "AsyncHTTPProtocolResourceWithRawResponse",
    "HTTPProtocolResourceWithStreamingResponse",
    "AsyncHTTPProtocolResourceWithStreamingResponse",
    "HTTPMethodResource",
    "AsyncHTTPMethodResource",
    "HTTPMethodResourceWithRawResponse",
    "AsyncHTTPMethodResourceWithRawResponse",
    "HTTPMethodResourceWithStreamingResponse",
    "AsyncHTTPMethodResourceWithStreamingResponse",
    "IPVersionResource",
    "AsyncIPVersionResource",
    "IPVersionResourceWithRawResponse",
    "AsyncIPVersionResourceWithRawResponse",
    "IPVersionResourceWithStreamingResponse",
    "AsyncIPVersionResourceWithStreamingResponse",
    "OSResource",
    "AsyncOSResource",
    "OSResourceWithRawResponse",
    "AsyncOSResourceWithRawResponse",
    "OSResourceWithStreamingResponse",
    "AsyncOSResourceWithStreamingResponse",
    "TLSVersionResource",
    "AsyncTLSVersionResource",
    "TLSVersionResourceWithRawResponse",
    "AsyncTLSVersionResourceWithRawResponse",
    "TLSVersionResourceWithStreamingResponse",
    "AsyncTLSVersionResourceWithStreamingResponse",
    "BrowserFamilyResource",
    "AsyncBrowserFamilyResource",
    "BrowserFamilyResourceWithRawResponse",
    "AsyncBrowserFamilyResourceWithRawResponse",
    "BrowserFamilyResourceWithStreamingResponse",
    "AsyncBrowserFamilyResourceWithStreamingResponse",
    "LocationsResource",
    "AsyncLocationsResource",
    "LocationsResourceWithRawResponse",
    "AsyncLocationsResourceWithRawResponse",
    "LocationsResourceWithStreamingResponse",
    "AsyncLocationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .locations import (
    LocationsResource,
    AsyncLocationsResource,
    LocationsResourceWithRawResponse,
    AsyncLocationsResourceWithRawResponse,
    LocationsResourceWithStreamingResponse,
    AsyncLocationsResourceWithStreamingResponse,
)
from .traffic_anomalies import (
    TrafficAnomaliesResource,
    AsyncTrafficAnomaliesResource,
    TrafficAnomaliesResourceWithRawResponse,
    AsyncTrafficAnomaliesResourceWithRawResponse,
    TrafficAnomaliesResourceWithStreamingResponse,
    AsyncTrafficAnomaliesResourceWithStreamingResponse,
)

__all__ = [
    "LocationsResource",
    "AsyncLocationsResource",
    "LocationsResourceWithRawResponse",
    "AsyncLocationsResourceWithRawResponse",
    "LocationsResourceWithStreamingResponse",
    "AsyncLocationsResourceWithStreamingResponse",
    "TrafficAnomaliesResource",
    "AsyncTrafficAnomaliesResource",
    "TrafficAnomaliesResourceWithRawResponse",
    "AsyncTrafficAnomaliesResourceWithRawResponse",
    "TrafficAnomaliesResourceWithStreamingResponse",
    "AsyncTrafficAnomaliesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["OrganizationListParams"]


class OrganizationListParams(TypedDict, total=False):
    direction: Literal["asc", "desc"]
    """Direction to order organizations."""

    match: Literal["any", "all"]
    """Whether to match all search requirements or at least one (any)."""

    name: str
    """Organization name."""

    order: Literal["id", "name", "status"]
    """Field to order organizations by."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of organizations per page."""

    status: Literal["member", "invited"]
    """Whether the user is a member of the organization or has an inivitation pending."""
